@@ -1,4 +1,4 @@
-use crate::{Message, RosMsg};
+use crate::{Message, RosMsg, ServicePair};
 use std::io;
 
 #[derive(Clone, Debug, Default, PartialEq)]
@@ -46,3 +46,8 @@ impl RosMsg for RawMessage {
         Ok(Self(data))
     }
 }
+
+impl ServicePair for RawMessage {
+    type Request = RawMessage;
+    type Response = RawMessage;
+}
