@@ -1,5 +1,4 @@
 use std::process::Command;
-use std::time;
 
 mod util;
 
@@ -33,7 +32,6 @@ fn client_to_inline_service() {
     })
     .unwrap();
 
-    rosrust::wait_for_service("add_two_ints", Some(time::Duration::from_secs(10))).unwrap();
     let client = rosrust::client::<msg::roscpp_tutorials::TwoInts>("add_two_ints").unwrap();
 
     test_request(&client, 0, 10);
